/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in the
 MANET_multicast directory) for conditions of use and redistribution.
*********************************************************************/

/*
Copyright (c) 1997, 1998 Carnegie Mellon University.  All Rights
Reserved. 

Permission to use, copy, modify, and distribute this
software and its documentation is hereby granted (including for
commercial or for-profit use), provided that both the copyright 
notice and this permission notice appear in all copies of the 
software, derivative works, or modified versions, and any portions 
thereof, and that both notices appear in supporting documentation, 
and that credit is given to Carnegie Mellon University in all 
publications reporting on direct or indirect use of this code or 
its derivatives.

ALL CODE, SOFTWARE, PROTOCOLS, AND ARCHITECTURES DEVELOPED BY THE CMU
MONARCH PROJECT ARE EXPERIMENTAL AND ARE KNOWN TO HAVE BUGS, SOME OF
WHICH MAY HAVE SERIOUS CONSEQUENCES. CARNEGIE MELLON PROVIDES THIS
SOFTWARE OR OTHER INTELLECTUAL PROPERTY IN ITS ``AS IS'' CONDITION,
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE OR
INTELLECTUAL PROPERTY, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

Carnegie Mellon encourages (but does not require) users of this
software or intellectual property to return any improvements or
extensions that they make, and to grant Carnegie Mellon the rights 
to redistribute these changes without encumbrance.

*/

/* 
 * The original AODV code was modified heavily to implement efficient 
 * broadcasting (BCAST) by Thomas Kunz.
 */

#include <bcast/bcast.h>
#include <bcast/bcast_packet.h>
#include <MANET_multicast/mttable.h>
#include <random.h>
#include <cmu-trace.h>

#define max(a,b)        a > b ? a : b
#define CURRENT_TIME    Scheduler::instance().clock()

//#define DEBUG
//#define ERROR

/*
  TCL Hooks
*/

int hdr_bcast::offset_;
static class BCASTHeaderClass : public PacketHeaderClass {
public:
        BCASTHeaderClass() : PacketHeaderClass("PacketHeader/BCAST",
                                              BCAST_HDR_LEN) {
		bind_offset(&hdr_bcast::offset_);
	} 
} class_rtProtoBCAST_hdr;

static class BCASTclass : public TclClass {
public:
        BCASTclass() : TclClass("Agent/BCAST") {}
        TclObject* create(int argc, const char*const* argv) {
          assert(argc == 5);
          return (new BCAST((nsaddr_t) atoi(argv[4])));
        }
} class_rtProtoBCAST;


int
BCAST::command(int argc, const char*const* argv) {
  if(argc == 2) {
  Tcl& tcl = Tcl::instance();
    
    if(strncasecmp(argv[1], "id", 2) == 0) {
      tcl.resultf("%d", index);
      return TCL_OK;
    }
    
    if(strncasecmp(argv[1], "start", 2) == 0) {
      htimer.handle((Event*) 0);
      ntimer.handle((Event*) 0);
      uid_handler.handle((Event*) 0);
      //bcast_queue.handle((Event*) 0);
      return TCL_OK;
     }               
  }
  else if(argc == 3) {
    if(strcmp(argv[1], "index") == 0) {
      index = atoi(argv[2]);
      return TCL_OK;
    }

    else if(strcmp(argv[1], "log-target") == 0 || strcmp(argv[1], "tracetarget") == 0) {
      logtarget = (Trace*) TclObject::lookup(argv[2]);
      if(logtarget == 0)
	return TCL_ERROR;
      return TCL_OK;
    }
    else if(strcmp(argv[1], "bcast-join-group")==0) {
      nsaddr_t mcast_addr = atoi(argv[2]);
      if (mcast_addr < IP_MULTICAST_RANGE) return TCL_ERROR;
				
      mt_entry *mt = mtable.mt_lookup(mcast_addr);
      if (!mt) mt = mtable.mt_add(mcast_addr);
      return TCL_OK;
    }
    else if(strcmp(argv[1], "bcast-leave-group")==0) {
      nsaddr_t mcast_addr = atoi(argv[2]);
      if (mcast_addr < IP_MULTICAST_RANGE) return TCL_ERROR;
				
      mt_entry *mt = mtable.mt_lookup(mcast_addr);
      if (!mt) mtable.mt_delete(mcast_addr);
      return TCL_OK;
    }
    else if(strcmp(argv[1], "if-queue") == 0) {
    ifqueue = (PriQueue*) TclObject::lookup(argv[2]);
      
      if(ifqueue == 0)
	return TCL_ERROR;
      return TCL_OK;
    }
    else if (strcmp(argv[1], "port-dmux") == 0) {
        dmux_ = (PortClassifier *)TclObject::lookup(argv[2]);
        if (dmux_ == 0) {
                fprintf (stderr, "%s: %s lookup of %s failed\n", __FILE__,
                argv[1], argv[2]);
                return TCL_ERROR;
        }
        return TCL_OK;
    }
  }
  return Agent::command(argc, argv);
}

/* 
   Constructor
*/

BCAST::BCAST(nsaddr_t id) : Agent(PT_BCAST), nacktimer(this), rtxtimer(this),
				htimer(this), ntimer(this),
				mtable(), bcast_queue(this), p_cache(CACHE_SIZE) {
                
  index = id;
  LIST_INIT(&nbhead);

  logtarget = 0;
  ifqueue = 0;
}

/*******************************************************************************
                                TIMERS
 *******************************************************************************/

void
NackTimer_bcast::handle(Event*) {
   // if we receibed the packet, silent discard the NACK
   if (seqno > 0 && agent->uid_handler.id_lookup(src, seqno)) 
	   return;
   // otherwise, issue a NACK (up to MAX_NACK times only)
   if (retry_count++ < MAX_NACK) {
#ifdef DEBUG
     fprintf(stderr, "Node %d: NACK timed out at %f, try again (%d)\n", 
       agent->index, CURRENT_TIME, retry_count);
#endif
     if (control_nack_flood()) {
#ifdef DEBUG
        fprintf(stderr, "Node %d: throttle nack\n", agent->index);
#endif
	return;
     }
     agent->sendNack(src, seqno);
     Scheduler::instance().schedule(this, &intr, NACK_TIMEOUT);
   }
   else {
     src = ILLEGAL_NODE; seqno = 0;
   }
}

void 
NackTimer_bcast::start_nack_timer(nsaddr_t p_src, 
		unsigned int p_seqno, double delay) {
   // there is a slight potential for trouble: assume we miss packets
   // N-2, N-1, and N from the same source. Upon receipt of packet N+1,
   // a NACK for packet N is issued. While we wait for packet N to be
   // retransmitted by a neighbour, due to the broadcast nature of the medium, 
   // we may overhear (and receive) retransmission of packet N-1 (requested
   // by another node in the neighborhood). This will trigger a NACK for N-2. 
   //
   // The code supports only a single outstanding NACK timeout. In this (rare)
   // case, our scheme fails: we will not set the NACK timer for N-2. If the
   // retransmission attempt of packet N-2 fails, we will not have a timeout to
   // recover.
   //
   if (intr.uid_ > 0) {
      // there is already a timer set, silently quit
      return;
   }
   src = p_src;
   seqno = p_seqno;
   retry_count = 0;
#ifdef DEBUG
   fprintf(stderr, "Node %d: starting NACK timer for packet (%d,%d)\n",
     agent->index, src, seqno);
#endif
   Scheduler::instance().schedule(this, &intr, delay);
}

void
NackTimer_bcast::cancel_nack_timer(nsaddr_t p_src, unsigned int p_seqno) {
   // check whether we had a NACK pending for this packet
   // and cancel
   if (intr.uid_) {
     if (p_src == src && p_seqno == seqno) {
#ifdef DEBUG
       fprintf(stderr, "Node %d: cancelling NACK timer for packet (%d,%d)\n",
         agent->index, src, seqno);
#endif
       Scheduler::instance().cancel(&intr);
       src = 0; seqno = 0;
     }
   }
}

// keep track of packet being sent or received to control NACK frequency

void
NackTimer_bcast::save_packet_time(double t_packet) { 
   packet_times[packet_ptr] = t_packet; 
   packet_ptr = (packet_ptr + 1) % PACKET_COUNT;
   if (packet_ptr == 0) count = PACKET_COUNT;
}

// if it is okay to send a NACK, return TRUE

int
NackTimer_bcast::control_nack_flood() {
// nack_ptr points to the next element to be overwritten, so
// this is the one the longest in the array...
if (count == PACKET_COUNT &&
    CURRENT_TIME - packet_times[packet_ptr] < MIN_DIFF) 
	return 0;
else
	return 1;
}

void
RetransmitTimer_bcast::handle(Event*) {
   // retransmit NACK-ed packet
#ifdef DEBUG
   fprintf(stderr, "Node %d: retransmit packet (%d,%d) at %f\n",
     agent->index, src, seqno, CURRENT_TIME);
#endif
   agent->sendPacket(p, 0.0);
   src = ILLEGAL_NODE; seqno = 0;
}

void 
RetransmitTimer_bcast::start_rtx_timer(Packet *pck, double delay) {
   if (intr.uid_ > 0) {
      // there is already a timer set, silently quit
      return;
   }
   // extract and save SRC and SEQNO for easier/faster comparison
   struct hdr_ip *ih = HDR_IP(pck);
   struct hdr_rtp *rh = HDR_RTP(pck);
   src = ih->saddr();
   seqno = rh->seqno_;
   // save a copy of the packet: by the time we retransmit, it may not
   // be in the node cache anymore....
   p = pck->copy();
   struct hdr_cmn *ch = HDR_CMN(p);
   ch->direction() = hdr_cmn::DOWN;
#ifdef DEBUG
   fprintf(stderr, "Node %d: RTX timer for packet (%d,%d) at %f, delay %f\n",
     agent->index, src, seqno, CURRENT_TIME, delay);
#endif
   Scheduler::instance().schedule(this, &intr, delay);
}

void
RetransmitTimer_bcast::cancel_rtx_timer(nsaddr_t p_src, unsigned int p_seqno) {
   // check whether we had a timer pending for this packet
   // and cancel
   if (intr.uid_) {
     if (p_src == src && p_seqno == seqno) {
#ifdef DEBUG
       fprintf(stderr, "Node %d: cancelling RTX timer for packet (%d,%d)\n",
         agent->index, src, seqno);
#endif
       Scheduler::instance().cancel(&intr);
       Packet::free(p); src = 0; seqno = 0;
     }
   }
}

void
HelloTimer_bcast::handle(Event*) {
   agent->sendHello();
   double interval = MinHelloInterval + 
                 ((MaxHelloInterval - MinHelloInterval) * Random::uniform());
   assert(interval >= 0);
   Scheduler::instance().schedule(this, &intr, interval);
}

void
HelloTimer_bcast::reschedule() {
   if (intr.uid_) {
#ifdef DEBUG
	fprintf(stderr, "Node %d: Reschedule hello message\n",
			agent->index);
#endif
	Scheduler::instance().cancel(&intr);
   }
   double interval = MinHelloInterval + 
                 ((MaxHelloInterval - MinHelloInterval) * Random::uniform());
   assert(interval >= 0);
   Scheduler::instance().schedule(this, &intr, interval);
}

void
NeighborTimer_bcast::handle(Event*) {
  agent->nb_purge();
  Scheduler::instance().schedule(this, &intr, HELLO_INTERVAL);
}

/*
  Packet Reception Routines
*/

void
BCAST::recv(Packet *p, Handler*) {
struct hdr_cmn *ch = HDR_CMN(p);
struct hdr_ip *ih = HDR_IP(p);
struct hdr_bcast *bh = HDR_BCAST(p);

 assert(initialized());

 if(ch->ptype() == PT_BCAST) {
   ih->ttl_ -= 1;
   recvBCAST(p);
   return;
 }

 /*
  *  Must be a packet I'm originating...
  */

 if((ih->saddr() == index) && (ch->num_forwards() == 0)) {
 /*
  * Add the IP Header
  */
   ch->size() += IP_HDR_LEN;
   ih->ttl_ = NETWORK_DIAMETER;
 }
 /*
  *  I received a packet that I sent.  Probably
  *  a routing loop.
  */
 else if(ih->saddr() == index) {
   drop(p, DROP_RTR_ROUTE_LOOP);
   return;
 }
 /*
  *  Packet I'm forwarding...
  */
 else {
 /*
  *  Check the TTL.  If it is zero, then discard.
  */
   if(--ih->ttl_ == 0) {
     drop(p, DROP_RTR_TTL);
     return;
   }
 }

#ifdef PIGGYBACK_HELLO
 // turn packet into BCASTTYPE_DATA packet so we can pihhyback HELLO msg
 ch->size() += BCAST_HDR_LEN;
 bh->old_type = ch->ptype(); // save old packet type
 ch->ptype() = PT_BCAST;
 bh->type = BCASTTYPE_DATA;
#endif

 // pass this packet on to be forwarded 
 forward(p);
}


void
BCAST::recvBCAST(Packet *p) {
struct hdr_bcast *bh = HDR_BCAST(p);
struct hdr_ip *ih = HDR_IP(p);

 assert(ih->sport() == RT_PORT);
 assert(ih->dport() == RT_PORT);

 // keep info about PACKET being received by node
 nacktimer.save_packet_time(CURRENT_TIME);

 /*
  * Incoming Packets.
  */
 switch(bh->type) {

 case BCASTTYPE_HELLO:
   recvHello(p);
   break;

 case BCASTTYPE_NACK:
   recvNack(p);
   break;

#ifdef PIGGYBACK_HELLO
 case BCASTTYPE_DATA:
   // Each packet of type BCAST_DATA has a HELLO message piggybacked, so
   // process this first
   BCAST_Neighbor *nb;
#ifdef DEBUG
   fprintf(stderr, "Node %d: processing piggybacked HELLO from %d\n",
	index, bh->dst);
#endif
   nb = nb_lookup(bh->dst);
   if(nb == 0) {
	nb_insert(bh->dst);
	nb = nb_lookup(bh->dst);
   }
   else {
	nb->nb_expire = CURRENT_TIME +
	(1.5 * ALLOWED_HELLO_LOSS * HELLO_INTERVAL);
   }
   // safe one-hop neighborhood information
   add_neighbors(nb, bh->no_of_neighbors, bh->neighbors);
   forward(p);
   break;
#endif

 default:
   fprintf(stderr, "Invalid BCAST type (%x)\n", bh->type);
   exit(1);
 }
}

/*
   Packet Transmission Routines
*/

void
BCAST::forward(Packet *p) {
struct hdr_cmn *ch = HDR_CMN(p);
struct hdr_ip *ih = HDR_IP(p);
struct hdr_rtp *rh = HDR_RTP(p);
#ifdef PIGGYBACK_HELLO
struct hdr_bcast *bh = HDR_BCAST(p);
#endif

// keep track of neighbors not covered
int no_of_neighbors;
nsaddr_t neighbors[NEIGHBOR_COUNT];

nsaddr_t src = ih->saddr();
unsigned int seqno = rh->seqno_;
mt_entry *mt;

int n, max;

 // keep track of packet reception to control NACK frequency
 nacktimer.save_packet_time(CURRENT_TIME);

 // See whether received packet (i.e. not originating from us) allows us 
 // to prune pending BCASTS.
 // We need to find the neighbourhood information for the sender and
 // process this against the information stored in each pending BCAST

 BCAST_Neighbor* nb;
 if (ih->saddr() != index && ((nb = nb_lookup(ch->prev_hop_)) != NULL)) {
#ifdef DEBUG
	fprintf(stderr, "Node %d: Processing neighbor info for neighbor %d\n",
		index, bh->src);
#endif
	bcast_queue.process(ih->saddr(), ch->uid(), nb->nb_no_of_neighbors,
		nb->nb_neighbors);
 }

 // check whether received packet is in response to a NACK and cancel
 // pending NACK timer
 nacktimer.cancel_nack_timer(src, seqno);

 // check whether received packet allows us to drop a pending retransmission
 rtxtimer.cancel_rtx_timer(src, seqno);

 if(ih->ttl_ == 0) {

#ifdef DEBUG
  fprintf(stderr, "Node %d: calling drop() (TTL expired)\n", index);
#endif // DEBUG
 
  drop(p, DROP_RTR_TTL);
  return;
 }

 if (uid_handler.id_lookup(src, seqno)) {
#ifdef DEBUG
   fprintf(stderr, "Node %d: discarding packet (not first time)\n", index);
#endif
   Packet::free(p);
   return;
 }

#ifdef RELIABLE
 // we also check whether we received previous packet (based on cached
 // unique IDs....)
 if (seqno > 0 && !uid_handler.id_lookup(src, seqno-1)) {
#ifdef DEBUG
   fprintf(stderr, "Node %d: missed packet (%d,%d) at %f, NACK it\n",
     index, src, seqno-1, CURRENT_TIME);
#endif
   if (nacktimer.control_nack_flood())
   	sendNack(src,seqno-1);
 }
#endif

 /*
  * Check whether this packet belongs to a multicast group 
  * this node subscribed to?
  */
 mt = mtable.mt_lookup(ih->daddr());
 if (mt) {

   // Need to make a copy of the packet
   Packet *p_new = p->copy();
   struct hdr_cmn *ch_new = HDR_CMN(p_new);
#ifdef PIGGYBACK_HELLO
   struct hdr_bcast *bh_new = HDR_BCAST(p);
#endif

   ch_new->addr_type() = NS_AF_INET;
   ch_new->direction() = hdr_cmn::UP;
   struct hdr_ip *ih_new = HDR_IP(p_new);

#ifdef PIGGYBACK_HELLO
   // "strip" BCAST header and make it a "regular" packet again
   ch_new->size() -= BCAST_HDR_LEN;
   ch_new->ptype() = bh_new->old_type;
#endif

   // "strip" the IP header...
   ch_new->size() -= IP_HDR_LEN;

   // send to "myself" at port 0 
   // (assume that receiving NULL agent sits there...)
   ih_new->daddr() = index;
   ih_new->dport() = 0;

   // pass packet to Link Layer, from where it will be passed up the stack to
   // agent at port 0 (routing agent cannot directly pass packets up the stack
   // unless we emulate the way DSR agents are linked in the protocol stack)
   Scheduler::instance().schedule(target_, p_new, 0.);
 } 

 /*
  * Cache the packet ID
  */
 uid_handler.id_insert(src, seqno);

 ch->addr_type() = NS_AF_ILINK; // allows us to force MAC broadcast 
 ch->direction() = hdr_cmn::DOWN; //important: change the packet's direction

 /* force MAC broadcast, bypassing ARP etc. Better than IP broadcast, which
    would override IP destination address */
 ch->next_hop_ = MAC_BROADCAST;

 /*
  * Cache the packet itself (neighbourhood information will be filled in
  * when the cached copy will be retransmitted, but it is important that
  * the direction etc. have been set, so the packet is ready to go....
  */
 p_cache.packet_insert(p);

 /*
  * determine and save my neighborhood information, minus sender of packet
  */
 int count = 0;
 nb = nbhead.lh_first;
 for(; nb; nb = nb->nb_link.le_next) {
	// skip sender of packet
	if (ch->prev_hop_ != nb->nb_addr) {
		neighbors[count++] = nb->nb_addr;
	}
 }
 no_of_neighbors = count;

 /*
  * if we originate that packet or do not have neighbor info
  * there is no need to process this further
  */
 if (ih->saddr() != index && ((nb = nb_lookup(ch->prev_hop_)) != NULL)) {
	// eliminate all neighbors that are also covered by sender...
#ifdef DEBUG
	fprintf(stderr, "Node %d: eliminate known neighbors of %d\n", 
		index, ch->prev_hop_);
#endif
	for (int i = 0; i < nb->nb_no_of_neighbors; i++) {
		for (int j = 0; j < no_of_neighbors; j++) {
			if (neighbors[j] == nb->nb_neighbors[i]) {
				for (int l = j+1; l < no_of_neighbors; l++) {
					neighbors[l-1] = neighbors[l];
				}
				no_of_neighbors--;
				break;
			}
		}
	}
 }

#ifdef DEBUG
	fprintf(stderr, "Node %d: remaining %d uncovered neighbours\n", 
		index, no_of_neighbors);
	for (int i = 0; i < no_of_neighbors; i++)
		fprintf(stderr, " %d", neighbors[i]);
	fprintf(stderr, "\n");
#endif

 ch->prev_hop_ = index; // mark current node as forwarder

 /* enqueue packet if there are uncovered neigbors */
 if (no_of_neighbors != 0) {
	// determine right amount of jitter
	nb_determine_degrees(&max, &n);
	bcast_queue.insert(p, no_of_neighbors, neighbors,
		CURRENT_TIME+((double)max/n) * BCAST_DELAY * Random::uniform());
 }
}

void
BCAST::sendPacket(Packet *p, double delay) {
 // we need to do 3 things:
 // 1) add neighbourhood info into Packet
 // 2) cancel and reschedule HELLO message
 // 3) send packet out
 struct hdr_cmn *ch = HDR_CMN(p);

#ifdef PIGGYBACK_HELLO
 struct hdr_bcast *bh = HDR_BCAST(p);
#endif

 // keep info about PACKET being sent by node
 nacktimer.save_packet_time(CURRENT_TIME);

#ifdef PIGGYBACK_HELLO
 // 1) fill in neighborhood information 
 // (i.e., piggyback info in HELLO message)
 int count = 0;
 BCAST_Neighbor *nb = nbhead.lh_first;
 for(; nb; nb = nb->nb_link.le_next) {
	bh->neighbors[count++] = nb->nb_addr;
 }
 bh->no_of_neighbors = count;

 if (count > NEIGHBOR_COUNT) {
	fprintf(stderr, "Node %d: too many neighbors (%d vs. max %d)\n", 
		index, count, NEIGHBOR_COUNT);
	exit(1);
 }

 // 2) check and cancel pending HELLO message, reschedule one for later
 htimer.reschedule();
#endif

 // mark current node as forwarder
 ch->prev_hop_ = index;

 // 3) send packet out
 Scheduler::instance().schedule(target_, p, delay);
}

/*
 * Dealing with retransmission requests
 */

void
BCAST::sendNack(nsaddr_t src, u_int32_t seqno) {
Packet *p = Packet::alloc();
struct hdr_cmn *ch = HDR_CMN(p);
struct hdr_ip *ih = HDR_IP(p);
struct hdr_bcast *bh = HDR_BCAST(p);

#ifdef DEBUG
 fprintf(stderr, "Node %d: sending NACK for (%d,%d) at %.2f\n", 
	index, src, seqno, Scheduler::instance().clock());
#endif

 // keep info about NACK being sent by node
 nacktimer.save_packet_time(CURRENT_TIME);

 bh->type = BCASTTYPE_NACK;
 bh->hop_count = 1;
 // ask for retransmission of packet from node ID with number BID
 bh->src = src;
 bh->src_seqno = seqno;

#ifdef PIGGYBACK_HELLO
 // piggyback neighbour information
 int count = 0;
 BCAST_Neighbor *nb = nbhead.lh_first;
 for(; nb; nb = nb->nb_link.le_next) {
	bh->neighbors[count++] = nb->nb_addr;
 }
 bh->no_of_neighbors = count;

 if (count > NEIGHBOR_COUNT) {
	fprintf(stderr, "Node %d: too many neighbors (%d vs. max %d)\n", 
		index, count, NEIGHBOR_COUNT);
	exit(1);
 }
 // check and cancel pending HELLO message, reschedule one for later
 htimer.reschedule();
#endif

 bh->dst = index;
 bh->lifetime = (1 + ALLOWED_HELLO_LOSS) * HELLO_INTERVAL;
 ch->ptype() = PT_BCAST;
 ch->size() = IP_HDR_LEN + sizeof(hdr_bcast);
 ch->iface() = -2;
 ch->error() = 0;
 ch->addr_type() = NS_AF_NONE;
 ch->prev_hop_ = index;

 ih->saddr() = index;
 ih->daddr() = IP_BROADCAST;
 ih->sport() = RT_PORT;
 ih->dport() = RT_PORT;
 ih->ttl_ = 1;

 /* send NACK, but add jitter..... */
 Scheduler::instance().schedule(target_, p, 0.01 * Random::uniform());
}

void
BCAST::recvNack(Packet *p) {
struct hdr_bcast *bh = HDR_BCAST(p);
struct hdr_cmn *ch = HDR_CMN(p);
BCAST_Neighbor *nb;
Packet *cached;
double delay;

#ifdef DEBUG
 fprintf(stderr, "Node %d: receiving NACK for (%d,%d) from %d at %.2f\n", 
	index, bh->src, bh->src_seqno, bh->dst, 
        Scheduler::instance().clock());
#endif

 /* see whether we have requested packet in cache and "queue" it for
    retransmission */
 cached = p_cache.packet_lookup(bh->src, bh->src_seqno);
 if (cached) {
   delay = 0.01 * Random::uniform();
#ifdef DEBUG
   fprintf(stderr, "Node %d: servicing NACK for (%d,%d) from %d at %.4f\n",
     index, bh->src, bh->src_seqno, bh->dst, 
     Scheduler::instance().clock() + delay);
#endif
   // sendPacket(cached->copy(), delay);
   rtxtimer.start_rtx_timer(cached, delay);
 }

#ifdef PIGGYBACK_HELLO
 /* extract piggyback'd neighbour information */
 nb = nb_lookup(ch->prev_hop_);
 if(nb == 0) {
   nb_insert(ch->prev_hop_);
   nb = nb_lookup(ch->prev_hop_);
 }
 else {
   nb->nb_expire = CURRENT_TIME +
                   (1.5 * ALLOWED_HELLO_LOSS * HELLO_INTERVAL);
 }
 // safe one-hop neighborhood information
 add_neighbors(nb, bh->no_of_neighbors, bh->neighbors);
#endif

 Packet::free(p);

#ifdef DEBUG
 fprintf(stderr, "Node %d: neighbours:\n", index);
 nb = nbhead.lh_first;

 for(; nb; nb = nb->nb_link.le_next) {
   fprintf(stderr," %d:", nb->nb_addr);
   for (int i = 0; i < nb->nb_no_of_neighbors; i++) {
	fprintf(stderr, " %d", nb->nb_neighbors[i]);
   }
   fprintf(stderr, "\n");
 }
#endif
}

/*
 * Neighbor Management Functions
 */

void
BCAST::sendHello() {
Packet *p = Packet::alloc();
struct hdr_cmn *ch = HDR_CMN(p);
struct hdr_ip *ih = HDR_IP(p);
struct hdr_bcast *bh = HDR_BCAST(p);

#ifdef DEBUG
 fprintf(stderr, "Node %d: sending Hello at %.2f\n", 
	index, Scheduler::instance().clock());
#endif

 // keep info about HELLO being sent by node
 nacktimer.save_packet_time(CURRENT_TIME);

 bh->type = BCASTTYPE_HELLO;
 bh->hop_count = 1;
 bh->dst = index;
 bh->lifetime = (1 + ALLOWED_HELLO_LOSS) * HELLO_INTERVAL;

 // fill in neighborhood information
 int count = 0;
 BCAST_Neighbor *nb = nbhead.lh_first;
 for(; nb; nb = nb->nb_link.le_next) {
	bh->neighbors[count++] = nb->nb_addr;
 }
 bh->no_of_neighbors = count;

 if (count > NEIGHBOR_COUNT) {
	fprintf(stderr, "Node %d: too many neighbors (%d vs. max %d)\n", 
		index, count, NEIGHBOR_COUNT);
	exit(1);
 }

 ch->ptype() = PT_BCAST;
 ch->size() = IP_HDR_LEN + sizeof(hdr_bcast);
 ch->iface() = -2;
 ch->error() = 0;
 ch->addr_type() = NS_AF_NONE;
 ch->prev_hop_ = index;

 ih->saddr() = index;
 ih->daddr() = IP_BROADCAST;
 ih->sport() = RT_PORT;
 ih->dport() = RT_PORT;
 ih->ttl_ = 1;

 Scheduler::instance().schedule(target_, p, 0.0);
}

void
BCAST::recvHello(Packet *p) {
struct hdr_cmn *ch = HDR_CMN(p);
struct hdr_bcast *bh = HDR_BCAST(p);
BCAST_Neighbor *nb;

 nb = nb_lookup(ch->prev_hop_);
 if(nb == 0) {
   nb_insert(ch->prev_hop_);
   nb = nb_lookup(ch->prev_hop_);
 }
 else {
   nb->nb_expire = CURRENT_TIME +
                   (1.5 * ALLOWED_HELLO_LOSS * HELLO_INTERVAL);
 }
 // safe one-hop neighborhood information
 add_neighbors(nb, bh->no_of_neighbors, bh->neighbors);

 Packet::free(p);

#ifdef DEBUG
 fprintf(stderr, "Node %d: neighbours:\n", index);
 nb = nbhead.lh_first;

 for(; nb; nb = nb->nb_link.le_next) {
   fprintf(stderr," %d:", nb->nb_addr);
   for (int i = 0; i < nb->nb_no_of_neighbors; i++) {
	fprintf(stderr, " %d", nb->nb_neighbors[i]);
   }
   fprintf(stderr, "\n");
 }
#endif
}

void
BCAST::nb_insert(nsaddr_t id) {
BCAST_Neighbor *nb = new BCAST_Neighbor(id);

 assert(nb);
 nb->nb_expire = CURRENT_TIME +
                (1.5 * ALLOWED_HELLO_LOSS * HELLO_INTERVAL);
 LIST_INSERT_HEAD(&nbhead, nb, nb_link);
}

BCAST_Neighbor*
BCAST::nb_lookup(nsaddr_t id) {
BCAST_Neighbor *nb = nbhead.lh_first;

 for(; nb; nb = nb->nb_link.le_next) {
   if(nb->nb_addr == id) break;
 }
 return nb;
}

void
BCAST::nb_determine_degrees(int *max, int *n) {
BCAST_Neighbor *nb = nbhead.lh_first;

 *n = 0;
 *max = 0;
 
 for(; nb; nb = nb->nb_link.le_next) {
	(*n)++;
	if (nb->nb_no_of_neighbors > *max) {
		*max = nb->nb_no_of_neighbors;
	}
 }
}
/*
 * Called when we receive *explicit* notification that a Neighbor
 * is no longer reachable.
 */
void
BCAST::nb_delete(nsaddr_t id) {
BCAST_Neighbor *nb = nbhead.lh_first;

 for(; nb; nb = nb->nb_link.le_next) {
   if(nb->nb_addr == id) {
     LIST_REMOVE(nb,nb_link);
     delete nb;
     break;
   }
 }
}


/*
 * Purges all timed-out Neighbor Entries - runs every
 * HELLO_INTERVAL * 1.5 seconds.
 */
void
BCAST::nb_purge() {
BCAST_Neighbor *nb = nbhead.lh_first;
BCAST_Neighbor *nbn;
double now = CURRENT_TIME;

 for(; nb; nb = nbn) {
   nbn = nb->nb_link.le_next;
   if(nb->nb_expire <= now) {
     nb_delete(nb->nb_addr);
   }
 }
}

/* 
 * Add 1-hop neighbourhood info for neighbour NB
 */
void
BCAST::add_neighbors(BCAST_Neighbor *nb, int num_neighbors,
					nsaddr_t *one_hop_neighbors) {
 for (int i = 0; i < num_neighbors; i++) {
	nb->nb_neighbors[i] = one_hop_neighbors[i];
 }
  nb->nb_no_of_neighbors = num_neighbors;
}
