# The driver for our simulations..

# ======================================================================
# Define options
# ======================================================================

set val(chan)       Channel/WirelessChannel
set val(prop)       Propagation/TwoRayGround
set val(netif)      Phy/WirelessPhy
set val(mac)        Mac/802_11
set val(ifq)        Queue/DropTail/PriQueue
set val(ll)         LL
set val(ant)        Antenna/OmniAntenna
set val(x)              1500   ;# X dimension of the topography
set val(y)              1500   ;# Y dimension of the topography
set val(ifqlen)         50            ;# max packet in ifq
set val(adhocRouting)   BCAST
set val(nn)             100             ;# how many nodes are simulated
set val(scDir)          "scenarios/" 
set val(stop)           910.0           ;# simulation time + 10 seconds

# minimize memory requirements by removing all irrelevant packet headers
remove-packet-header PGM PGM_SPM PGM_NAK Pushback NV LDP MPLS rtProtoLS Ping
remove-packet-header TFRC TFRC_ACK Diffusion RAP TORA IMEP MIP
remove-packet-header IPinIP Encap HttpInval MFTP SRMEXT SRM aSRM
remove-packet-header mcastCtrl CtrMcast rtProtoDV GAF Snoop TCPA TCP IVS
remove-packet-header Resv UMP Src_rt

# selectively allow only one of the following: AODV, SR, BCAST
remove-packet-header AODV
remove-packet-header SR
# remove-packet-header BCAST

# =====================================================================
# Main Program
# ======================================================================

#
# Check for command-line parameters
#

if {$argc != 4} {
	error "Not enough command-line parameters"
}
set val(seed) [lindex $argv 3].0

#
# Initialize Global Variables
#

# create simulator instance

set ns_		[new Simulator]

# setup topography object

set topo	[new Topography]

# create trace object for ns

set tracefd	[open [lindex $argv 2] w]
# set namtrace	[open namtrace.nam w]

# $ns_ use-newtrace
$ns_ trace-all $tracefd
# $ns_ namtrace-all-wireless $namtrace $val(x) $val(y)

# define topology
$topo load_flatgrid $val(x) $val(y)

#
# Create God
#
set god_ [create-god $val(nn)]

#
# define how node should be created
#

#global node setting

$ns_ node-config -adhocRouting $val(adhocRouting) \
                 -llType $val(ll) \
                 -macType $val(mac) \
                 -ifqType $val(ifq) \
                 -ifqLen $val(ifqlen) \
                 -antType $val(ant) \
                 -propType $val(prop) \
                 -phyType $val(netif) \
                 -channelType $val(chan) \
		 -topoInstance $topo \
		 -agentTrace ON \
                 -routerTrace OFF \
                 -macTrace ON 

#
#  Create the specified number of nodes [$val(nn)] and "attach" them
#  to the channel. 

for {set i 0} {$i < $val(nn) } {incr i} {
	set node_($i) [$ns_ node]	
	$node_($i) random-motion 0		;# disable random motion
# create a default target for redirecting the traffic (will be at port 0)
	set null_($i) [new Agent/Null]
	$ns_ attach-agent $node_($i) $null_($i)
}

# 
# Define node movement model
#
puts "Loading connection pattern..."
source [lindex $argv 0]

# 
# Define traffic model
#
puts "Loading scenario file..."
source $val(scDir)[lindex $argv 1]

#
# Tell nodes when the simulation ends
#
for {set i 0} {$i < $val(nn) } {incr i} {
    $ns_ at $val(stop).0 "$node_($i) reset";
}

$ns_ at  $val(stop).0002 "puts \"NS EXITING...\" ; $ns_ halt"

puts $tracefd "M 0.0 nn $val(nn) x $val(x) y $val(y) rp $val(adhocRouting)"
puts $tracefd "M 0.0 [lindex $argv 0]"
puts $tracefd "M 0.0 sc $val(scDir)[lindex $argv 1]"
puts $tracefd "M 0.0 [lindex $argv 2]"
puts $tracefd "M 0.0 seed $val(seed) prop $val(prop) ant $val(ant)"

puts "Starting Simulation..."
$ns_ run


