#!/bin/bash
#
# Invoke a serious of runs of NS2. Scenario files come from the
# SCENARIOS subdirectory, traffic files from TRAFFIC, with tracefiles
# being written into the RESULTS subdirectory. The command-line parameters
# are the filenames within these directories for traffic, scenario, and
# result, in that order, followed by the initial random number seed.
#
# Parameters: $1 = traffic subdirectory
#             $2 = result subdirectory
#             $3 = max scenario file speed (1 m/s or 20 m/s)
#
# example: NS DRIVER.TCL TRAFFIC SCENARIO RESULT SEED
#
# Naming conventions: all scenario files have names that denote 
# number of nodes, maximum speed, pause time,and a running number:
# SCEN-NODE-MAX-PAUSE-NUMBER
#
# All traffic files have names that denote the protocol,
# the number of senders, and the number of receivers:
# FLOOD-SENDERS-RECEIVERS
#
# The resulting trace files have names that are similar to
# the traffic files, but with in indication of the max speed
# and an additional running number:
# FLOOD<MAX>-SENDERS-RECEIVERS-NUMBER
#
if (( $# != 3 )); then
	echo "Usage: $0 traffic_dir result_dir speed"
	exit
else
	echo "batch: $1 $2 $3"
fi
#
# SN = number of senders
# RN = number of receivers
#
for sn in 01 02 05 10; do
  for rn in 010 020 040 060 080 100; do
    echo "$sn Senders with $rn Receivers"
      for i in 01 02 03 04 05 06 07 08 09 10; do
	echo "ns driver.flood.tcl $1/flood-$sn-$rn scen-100-$3-0-$i $2/flood$3-$sn-$rn-$i 2"
	ns driver.flood.tcl $1/flood-$sn-$rn scen-100-$3-0-$i $2/flood$3-$sn-$rn-$i 2
	awk -f eval.awk < $2/flood$3-$sn-$rn-$i > $2/summary$3-$sn-$rn-$i
	rm $2/flood$3-$sn-$rn-$i
      done
    done
done
